%letter_test1
clear all;
close all;

% Modify the parameters below to specify how many test runs to do
% and how much noise to add, i.e. how many neurons to 'flip'

num_test_runs   =   1000;
num_flips       =   2;

memories_to_use = ['D','M'];  %choose between 'D', 'M', 'J', and 'C'

%load data
D = [1 0 0 0 1 1 0 1 1 0 1 0 1 1 0 1 0 1 1 0 1 0 0 0 1];
J = [0 0 0 0 0 1 1 1 0 1 1 1 1 0 1 0 1 1 0 1 0 0 0 1 1];
C = [1 0 0 0 0 0 1 1 1 1 0 1 1 1 1 0 1 1 1 1 1 0 0 0 0];
M = [0 1 1 1 0 0 0 1 0 0 0 1 0 1 0 0 1 1 1 0 0 1 1 1 0];

%create data matrix
memory_options = 'DJCM';
for which_memory = 1:length(memories_to_use)
    switch find(memory_options==repmat(memories_to_use(which_memory),1,length(memory_options)))
        case 1
            data(which_memory,:) = D;
        case 2
            data(which_memory,:) = J;
        case 3
            data(which_memory,:) = C;
        case 4
            data(which_memory,:) = M;
        otherwise
            error('Memory option does not match one of saved memories.');
    end;
end;

n = 25; %number of data points in each picture
%make all 0s in data into -1s
datamones = data - ones(size(data));
newdata = data + datamones;

%make weights matrix (n x n)
w = zeros(n,n);
for i = 1:n
    for j = 1:n
        if i == j
            w(i,j) = 0;
        else
            w(i,j) = newdata(:,i)'*newdata(:,j);
        end
    end
end

num_memories = size(newdata,1);

for run_num = 1:num_test_runs,
    %initial state
    Xinit = newdata(randi(num_memories),:);
    
    %initial state (with errors)
    temp  = ones(1,n);
    which_to_flip   =   randperm(n,num_flips);
    temp(which_to_flip) = -1;
    Xerr  = Xinit.*temp;
    
    %stable state
    Xerror = Xerr;
    Xtest = Xerror;
    X_new = Xtest;
    X_old = ones(size(Xtest));
    count = 0;
    while X_old*X_new'~=length(Xtest)
        count = count +1;
        % step 3: Asynchonous update
        if count == 50
            break
        else
            X_old = X_new; % previous new is now old
            X_new = AUpdate(X_old,w);
        end
    end
    
    match_level(run_num) = (Xinit*X_new')/n;
end;

average_per_match   = mean((match_level+1)/2*100);
num_exact_match = sum(match_level >= .999);
num_exact_opposite = sum(match_level <= -.999);

percent_match       =   num_exact_match/num_test_runs*100;
percent_opposite    =   num_exact_opposite/num_test_runs*100;

home;
display(sprintf('Number of flips: %1.0f, Number of runs: %1.0f', num_flips, num_test_runs));
display(sprintf('Average percent overlap of neurons: %1.1f',average_per_match));
display(sprintf('Percent exact match for all neurons: %1.1f',percent_match));
display(sprintf('Percent exact opposite for all neurons: %1.1f',percent_opposite));